#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
import argparse as ap
import datetime
import logging
import sys

import pandas as pd
from pyat import version
from pyat.schema import create_engine, create_session
from pyat.schema.model import Filegroup
from pycapo import CapoConfig
from sqlalchemy import create_engine
from sqlalchemy.orm import (
    sessionmaker,  # an Engine, which the Session will use for connection
)

_NOW = datetime.datetime.utcnow()
_LOCATION = "DSOC"
_FILE_TYPE = "MARK4"
_MAX_ROWS_TO_IMPORT = 2

_LOG = logging.getLogger("audit")
_LOG.setLevel(logging.INFO)
ch = logging.StreamHandler(sys.stdout)
formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
ch.setFormatter(formatter)
ch.setLevel(logging.DEBUG)
_LOG.addHandler(ch)
_DESCRIPTION = """VLBA Audit version {}."""


def audit_vlba():
    """
    Find matching idifits / mark4 pairs in the legacy archive. Create a mark4 file entry in the
    new archive and attach it to the same filegroup as the idifits file.
    :return:
    """

    args = _make_parser().parse_args()

    ngas_session = create_session("NGAS", profile=args.profile)
    ngas_engine = create_engine("NGAS", profile=args.profile)

    ngas_vlba_files = pd.read_sql(
        """
            select f.file_id filename
            from ngas_files f
            where f.file_id like 'VLBA%%'
            and f.file_id not like '%%difx.log'
            and f.file_id not like '%%mark4%%'
            """,
        ngas_engine,
    )

    ngas_session.close()
    _LOG.info(f"{len(ngas_vlba_files)} VLBA files in NGAS")

    # legacy_engine = create_engine('LEGACY', profile=args.profile)
    # legacy_session = create_session('LEGACY', profile=args.profile)
    # legacy_vlba_files = pd.read_sql("""
    #         select arch_file filename
    #         from archive
    #         where arch_file like 'VLBA%%'
    #         """,
    #                                     legacy_engine)
    #
    # legacy_session.close()
    #
    # _LOG.info(f"{len(legacy_vlba_files)} VLBA files in legacy archive")

    archive_engine = create_engine("SDM", profile=args.profile)
    archive_session = create_session("SDM", profile=args.profile)

    archive_vlba_files = pd.read_sql(
        """
            select filename
            from files
            where filename like 'VLBA%%'
            """,
        archive_engine,
    )

    archive_session.close()

    _LOG.info(f"{len(archive_vlba_files)} VLBA files in new archive")

    # file_merge = legacy_vlba_files.merge(ngas_vlba_files, indicator=True, how='outer')
    # file_merge = legacy_vlba_files.merge(ngas_vlba_files, indicator=True, how='outer')
    file_merge = ngas_vlba_files.merge(archive_vlba_files, indicator=True, how="outer")
    diff = file_merge[file_merge["_merge"] == "left_only"]

    _LOG.info(f"{len(diff)} unmatching entries")
    _LOG.info(diff["filename"].head(25))


def _make_parser():
    r"""Build a command line parser for this app: this is external to MrBooks and/or CapoSettings because both need it,
    and Sphinx will want a look at ot to build docs."""
    result = ap.ArgumentParser(description=_DESCRIPTION.format(version), formatter_class=ap.RawTextHelpFormatter)
    result.add_argument("-P", "--profile", action="store", help="profile name to use, e.g. test, production")
    return result


if __name__ == "__main__":
    audit_vlba()
